@@ -1,3 +1,48 @@
+/*
+ * The olsr.org Optimized Link-State Routing daemon (olsrd)
+ *
+ * (c) by the OLSR project
+ *
+ * See our Git repository to find out who worked on this file
+ * and thus is a copyright holder on it.
+ *
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * * Redistributions of source code must retain the above copyright
+ *   notice, this list of conditions and the following disclaimer.
+ * * Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in
+ *   the documentation and/or other materials provided with the
+ *   distribution.
+ * * Neither the name of olsr.org, olsrd nor the names of its
+ *   contributors may be used to endorse or promote products derived
+ *   from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Visit http://www.olsr.org for more information.
+ *
+ * If you find this software useful feel free to make a donation
+ * to the project. For more information see the website or contact
+ * the copyright holders.
+ *
+ */
+
 #include <OlsrdPudWireFormat/wireFormat.h>
 
 #include <time.h>
@@ -22,7 +67,7 @@ bool isValidNodeIdType(unsigned long long nodeIdType) {
  */
 
 /** Determine the validity time in seconds from the OLSR wire format value */
-#define PUD_VALIDITY_TIME_FROM_OLSR(msn, lsn) ((((lsn) + 16) * (1 << (msn))) - 16)
+#define PUD_VALIDITY_TIME_FROM_OLSR(msn, lsn) ((((lsn) + 16) * (1u << (msn))) - 16)
 
 /**
  Get the validity time from a message
@@ -61,7 +106,7 @@ void setValidityTime(uint8_t * validityTimeField, unsigned long long validityTim
 		lsn = 15;
 	} else {
 		unsigned long lowerBound = PUD_VALIDITY_TIME_FROM_OLSR(msn, 0);
-		unsigned long resolution = (1 << msn);
+		unsigned long resolution = (1u << msn);
 		lsn = ((validityTime - lowerBound + (resolution >> 1)) / resolution);
 	}
 
@@ -369,7 +414,7 @@ double getPositionUpdateLatitude(
 	/* take half of the rounding error */
 	lat += 0.5;
 
-	lat /= (double) (1 << PUD_LATITUDE_BITS);
+	lat /= (double) (1u << PUD_LATITUDE_BITS);
 	/* lat is now in [0, 1> */
 
 	lat -= 0.5;
@@ -403,12 +448,12 @@ void setPositionUpdateLatitude(
 	lat += 0.5;
 	/* lat is now in [0, 1] */
 
-	lat *= (double) (1 << PUD_LATITUDE_BITS);
+	lat *= (double) (1u << PUD_LATITUDE_BITS);
 	/* lat is now in [0, LATITUDE_BITS] */
 
 	/* clip max */
-	if (unlikely(lat > (double)((1 << PUD_LATITUDE_BITS) - 1))) {
-		lat = (double) ((1 << PUD_LATITUDE_BITS) - 1);
+	if (unlikely(lat > (double)((1u << PUD_LATITUDE_BITS) - 1))) {
+		lat = (double) ((1u << PUD_LATITUDE_BITS) - 1);
 	}
 	/* lat is now in [0, 2^LATITUDE_BITS> */
 
@@ -434,7 +479,7 @@ double getPositionUpdateLongitude(
 	/* take half of the rounding error */
 	lon += 0.5;
 
-	lon /= (1 << PUD_LONGITUDE_BITS);
+	lon /= (1u << PUD_LONGITUDE_BITS);
 	/* lon is now in [0, 1> */
 
 	lon -= 0.5;
@@ -468,12 +513,12 @@ void setPositionUpdateLongitude(
 	lon += 0.5;
 	/* lon is now in [0, 1] */
 
-	lon *= (double) (1 << PUD_LONGITUDE_BITS);
+	lon *= (double) (1u << PUD_LONGITUDE_BITS);
 	/* lon is now in [0, LONGITUDE_BITS] */
 
 	/* clip max */
-	if (unlikely(lon > (double)((1 << PUD_LATITUDE_BITS) - 1))) {
-		lon = (double) ((1 << PUD_LATITUDE_BITS) - 1);
+	if (unlikely(lon > (double)((1u << PUD_LATITUDE_BITS) - 1))) {
+		lon = (double) ((1u << PUD_LATITUDE_BITS) - 1);
 	}
 
 	/* lon is now in [0, 2^LONGITUDE_BITS> */
