@@ -2,13 +2,11 @@ package aQute.bnd.repository.maven.provider;
 
 import java.io.Closeable;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
-import java.lang.reflect.InvocationTargetException;
 import java.net.URI;
 import java.net.URLDecoder;
 import java.util.ArrayList;
@@ -23,24 +21,25 @@ import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeSet;
 import java.util.concurrent.Callable;
-import java.util.concurrent.Executor;
 import java.util.concurrent.ScheduledFuture;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.jar.Manifest;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
 
 import org.osgi.resource.Capability;
 import org.osgi.resource.Requirement;
+import org.osgi.util.promise.Failure;
 import org.osgi.util.promise.Promise;
+import org.osgi.util.promise.Success;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import aQute.bnd.annotation.plugin.BndPlugin;
+import aQute.bnd.build.Project;
 import aQute.bnd.build.Workspace;
 import aQute.bnd.header.Attrs;
 import aQute.bnd.header.Parameters;
 import aQute.bnd.http.HttpClient;
-import aQute.bnd.jpm.util.JSONRPCProxy;
 import aQute.bnd.maven.PomResource;
 import aQute.bnd.osgi.Constants;
 import aQute.bnd.osgi.FileResource;
@@ -60,9 +59,10 @@ import aQute.bnd.service.RepositoryListenerPlugin;
 import aQute.bnd.service.RepositoryPlugin;
 import aQute.bnd.service.maven.PomOptions;
 import aQute.bnd.service.maven.ToDependencyPom;
+import aQute.bnd.service.release.ReleaseBracketingPlugin;
 import aQute.bnd.version.Version;
-import aQute.jpm.facade.repo.JpmRepo;
 import aQute.lib.converter.Converter;
+import aQute.lib.exceptions.Exceptions;
 import aQute.lib.hex.Hex;
 import aQute.lib.io.IO;
 import aQute.libg.cryptography.SHA1;
@@ -70,7 +70,6 @@ import aQute.libg.glob.Glob;
 import aQute.maven.api.Archive;
 import aQute.maven.api.IMavenRepo;
 import aQute.maven.api.IPom;
-import aQute.maven.api.Program;
 import aQute.maven.api.Release;
 import aQute.maven.api.Revision;
 import aQute.maven.provider.MavenBackingRepository;
@@ -82,18 +81,17 @@ import aQute.service.reporter.Reporter;
  * This is the Bnd repository for Maven.
  */
 @BndPlugin(name = "MavenBndRepository")
-public class MavenBndRepository extends BaseRepository
-		implements RepositoryPlugin, RegistryPlugin, Plugin, Closeable, Refreshable, Actionable, ToDependencyPom {
+public class MavenBndRepository extends BaseRepository implements RepositoryPlugin, RegistryPlugin, Plugin, Closeable,
+		Refreshable, Actionable, ToDependencyPom, ReleaseBracketingPlugin {
+	private final static Logger		logger						= LoggerFactory.getLogger(MavenBndRepository.class);
 
-	private final Pattern			JPM_REVISION_URL_PATTERN_P	= Pattern
-			.compile("https?://.+#!?/p/sha/(?<sha>([0-9A-F][0-9A-F]){20,20})/.*", Pattern.CASE_INSENSITIVE);
 	private static final String		NONE						= "NONE";
 	static final String				MAVEN_REPO_LOCAL			= System.getProperty("maven.repo.local",
 			"~/.m2/repository");
 	private Configuration			configuration;
 	private Registry				registry;
 	private File					localRepo;
-	Reporter						reporter;
+	private Reporter				reporter;
 	IMavenRepo						storage;
 	private boolean					inited;
 	private boolean					ok							= true;
@@ -101,18 +99,29 @@ public class MavenBndRepository extends BaseRepository
 	private ScheduledFuture< ? >	indexPoller;
 	private RepoActions				actions						= new RepoActions(this);
 	private String					name;
-	private JpmRepo					libraryx;
 	private HttpClient				client;
+	private ReleasePluginImpl		releasePlugin		= new ReleasePluginImpl(this, null);
+
+	static class LocalPutResult extends PutResult {
+		Archive			binaryArchive;
+		PutOptions		options;
+		String			failed;
+		public Archive	pomArchive;
+	}
 
 	@Override
 	public PutResult put(InputStream stream, PutOptions options) throws Exception {
 		init();
 		File binaryFile = File.createTempFile("put", ".jar");
 		File pomFile = File.createTempFile("pom", ".xml");
+		LocalPutResult result = new LocalPutResult();
 		try {
-			PutResult result = new PutResult();
+
 			if (options == null)
 				options = new PutOptions();
+			else {
+				result.options = options;
+			}
 
 			IO.copy(stream, binaryFile);
 
@@ -132,7 +141,7 @@ public class MavenBndRepository extends BaseRepository
 
 			ReleaseDTO instructions = getReleaseDTO(options.context);
 
-			try (Jar binary = new Jar(binaryFile);) {
+			try (Jar binary = new Jar(binaryFile)) {
 				Resource pomResource;
 
 				if (instructions.pom.path != null) {
@@ -155,46 +164,48 @@ public class MavenBndRepository extends BaseRepository
 
 				IO.copy(pomResource.openInputStream(), pomFile);
 				IPom pom;
-				try (FileInputStream fin = new FileInputStream(pomFile)) {
+				try (InputStream fin = IO.stream(pomFile)) {
 					pom = storage.getPom(fin);
 				}
 				Archive binaryArchive = pom.binaryArchive();
 
 				checkRemotePossible(instructions, binaryArchive.isSnapshot());
 
-				if (!binaryArchive.isSnapshot() && storage.exists(binaryArchive)) {
-					reporter.trace("Alrady released %s", pom.getRevision());
-					result.alreadyReleased = true;
-					return result;
+				if (!binaryArchive.isSnapshot()) {
+					releasePlugin.add(options.context, pom);
+					if (storage.exists(binaryArchive)) {
+						logger.debug("Already released {} to {}", pom.getRevision(), this);
+						result.alreadyReleased = true;
+						return result;
+					}
 				}
 
-				reporter.trace("Put release %s", pom.getRevision());
-				try (Release releaser = storage.release(pom.getRevision(), options.context.getProperties());) {
+				logger.debug("Put release {}", pom.getRevision());
+				try (Release releaser = storage.release(pom.getRevision(), options.context.getProperties())) {
 					if (releaser == null) {
-						reporter.trace("already released %s", pom.getRevision());
+						logger.debug("Already released {}", pom.getRevision());
 						return result;
 					}
 					if (instructions.snapshot >= 0)
-						releaser.setBuild(instructions.snapshot, "1");
+						releaser.setBuild(instructions.snapshot, null);
 
 					if (isLocal(instructions))
 						releaser.setLocalOnly();
 
 					releaser.add(pom.getRevision().pomArchive(), pomFile);
 					releaser.add(binaryArchive, binaryFile);
-
-					result.artifact = isLocal(instructions) ? storage.toLocalFile(binaryArchive).toURI()
-							: storage.toRemoteURI(binaryArchive);
+					result.binaryArchive = binaryArchive;
+					result.pomArchive = pom.getRevision().pomArchive();
 
 					if (!isLocal(instructions)) {
 
-						try (Tool tool = new Tool(options.context, binary);) {
+						try (Tool tool = new Tool(options.context, binary)) {
 
 							if (instructions.javadoc != null) {
 								if (!NONE.equals(instructions.javadoc.path)) {
 									try (Jar jar = getJavadoc(tool, options.context, instructions.javadoc.path,
 											instructions.javadoc.options,
-											instructions.javadoc.packages == JavadocPackages.EXPORT);) {
+											instructions.javadoc.packages == JavadocPackages.EXPORT)) {
 										save(releaser, pom.getRevision(), jar, "javadoc");
 									}
 								}
@@ -202,7 +213,7 @@ public class MavenBndRepository extends BaseRepository
 
 							if (instructions.sources != null) {
 								if (!NONE.equals(instructions.sources.path)) {
-									try (Jar jar = getSource(tool, options.context, instructions.sources.path);) {
+									try (Jar jar = getSource(tool, options.context, instructions.sources.path)) {
 										save(releaser, pom.getRevision(), jar, "sources");
 									}
 								}
@@ -210,13 +221,16 @@ public class MavenBndRepository extends BaseRepository
 						}
 					}
 				}
-				if (!binaryArchive.isSnapshot())
+				if (configuration.noupdateOnRelease() == false && !binaryArchive.isSnapshot())
 					index.add(binaryArchive);
 			}
 			return result;
+		} catch (Exception e) {
+			result.failed = e.getMessage();
+			throw e;
 		} finally {
-			binaryFile.delete();
-			pomFile.delete();
+			IO.delete(binaryFile);
+			IO.delete(pomFile);
 		}
 
 	}
@@ -276,7 +290,7 @@ public class MavenBndRepository extends BaseRepository
 			jar.write(tmp);
 			releaser.add(revision.archive(extension, classifier), tmp);
 		} finally {
-			tmp.delete();
+			IO.delete(tmp);
 		}
 
 	}
@@ -289,7 +303,7 @@ public class MavenBndRepository extends BaseRepository
 		if (context == null)
 			return release;
 
-		Parameters p = new Parameters(context.getProperty(Constants.MAVEN_RELEASE));
+		Parameters p = new Parameters(context.getProperty(Constants.MAVEN_RELEASE), reporter);
 
 		release.type = storage.isLocalOnly() ? ReleaseType.LOCAL : ReleaseType.REMOTE;
 
@@ -351,7 +365,7 @@ public class MavenBndRepository extends BaseRepository
 		if (manifest == null)
 			return null;
 
-		try (Processor scoped = context == null ? new Processor() : new Processor(context);) {
+		try (Processor scoped = context == null ? new Processor() : new Processor(context)) {
 			if (scoped.getProperty(Constants.GROUPID) == null)
 				scoped.setProperty(Constants.GROUPID, "osgi-bundle");
 			return new PomResource(scoped, manifest);
@@ -389,7 +403,35 @@ public class MavenBndRepository extends BaseRepository
 			if (listeners.length == 0)
 				return promise.getValue();
 
-			promise.onResolve(getResolveable(file, promise, listeners));
+			promise.then(new Success<File,Void>() {
+				@Override
+				public Promise<Void> call(Promise<File> resolved) throws Exception {
+					File value = resolved.getValue();
+					if (value == null) {
+						throw new FileNotFoundException("Download failed");
+					}
+					for (DownloadListener dl : listeners) {
+						try {
+							dl.success(value);
+						} catch (Exception e) {
+							reporter.exception(e, "Download listener failed in success callback %s", dl);
+						}
+					}
+					return null;
+				}
+			}).then(null, new Failure() {
+				@Override
+				public void fail(Promise< ? > resolved) throws Exception {
+					String reason = Exceptions.toString(resolved.getFailure());
+					for (DownloadListener dl : listeners) {
+						try {
+							dl.failure(file, reason);
+						} catch (Exception e) {
+							reporter.exception(e, "Download listener failed in failure callback %s", dl);
+						}
+					}
+				}
+			});
 			return file;
 		}
 		return null;
@@ -443,12 +485,11 @@ public class MavenBndRepository extends BaseRepository
 
 			localRepo = IO.getFile(configuration.local(MAVEN_REPO_LOCAL));
 
-			Executor executor = registry.getPlugin(Executor.class);
 			List<MavenBackingRepository> release = MavenBackingRepository.create(configuration.releaseUrl(), reporter,
 					localRepo, client);
 			List<MavenBackingRepository> snapshot = MavenBackingRepository.create(configuration.snapshotUrl(), reporter,
 					localRepo, client);
-			storage = new MavenRepository(localRepo, getName(), release, snapshot, executor, reporter,
+			storage = new MavenRepository(localRepo, getName(), release, snapshot, Processor.getExecutor(), reporter,
 					getRefreshCallback());
 
 			File base = IO.work;
@@ -461,11 +502,10 @@ public class MavenBndRepository extends BaseRepository
 			File indexFile = IO.getFile(base, configuration.index(name.toLowerCase() + ".mvn"));
 			IndexFile ixf = new IndexFile(reporter, indexFile, storage);
 			ixf.open();
-			ixf.sync();
 			this.index = ixf;
 			startPoll(index);
 
-			reporter.trace("Maven Bnd repository initing %s", this);
+			logger.debug("initing {}", this);
 		} catch (Exception e) {
 			reporter.exception(e, "Init for maven repo failed %s", configuration);
 			throw new RuntimeException(e);
@@ -473,6 +513,12 @@ public class MavenBndRepository extends BaseRepository
 	}
 
 	private void startPoll(final IndexFile index) {
+		Workspace ws = registry.getPlugin(Workspace.class);
+		if ((ws != null) && (ws.getGestalt().containsKey(Constants.GESTALT_BATCH)
+				|| ws.getGestalt().containsKey(Constants.GESTALT_CI)
+				|| ws.getGestalt().containsKey(Constants.GESTALT_OFFLINE))) {
+			return;
+		}
 		final AtomicBoolean busy = new AtomicBoolean();
 		indexPoller = Processor.getScheduledExecutor().scheduleAtFixedRate(new Runnable() {
 
@@ -523,52 +569,11 @@ public class MavenBndRepository extends BaseRepository
 
 	@Override
 	public void close() throws IOException {
-		if (storage != null)
-			storage.close();
+		IO.close(storage);
 		if (indexPoller != null)
 			indexPoller.cancel(true);
 	}
 
-	private Runnable getResolveable(final File file, final Promise<File> promise, final DownloadListener... listeners) {
-		return new Runnable() {
-
-			@Override
-			public void run() {
-				try {
-					if (promise.getFailure() == null) {
-						doSuccess(promise, listeners);
-					} else {
-						doFailure(file, promise, listeners);
-					}
-				} catch (Exception e) {
-					// can't happen, we checked
-					throw new RuntimeException(e);
-				}
-			}
-
-			void doFailure(final File file, final Promise<File> promise, final DownloadListener... listeners) {
-				for (DownloadListener dl : listeners)
-					try {
-						dl.failure(file, promise.getFailure().getMessage());
-					} catch (Exception e) {
-						reporter.exception(e, "Download listener failed in failure callback %s", dl);
-					}
-			}
-
-			void doSuccess(final Promise<File> promise, final DownloadListener... listeners)
-					throws InvocationTargetException, InterruptedException {
-				File file = promise.getValue();
-				for (DownloadListener dl : listeners)
-					try {
-						dl.success(file);
-					} catch (Exception e) {
-						reporter.exception(e, "Download listener failed in success callback %s", dl);
-					}
-			}
-
-		};
-	}
-
 	private Callable<Boolean> getRefreshCallback() {
 		return new Callable<Boolean>() {
 
@@ -670,7 +675,7 @@ public class MavenBndRepository extends BaseRepository
 		try {
 			return remote.getUser();
 		} catch (Exception e) {
-			return "error: " + e.getMessage();
+			return "error: " + e.toString();
 		}
 	}
 
@@ -722,7 +727,7 @@ public class MavenBndRepository extends BaseRepository
 		int n = t.indexOf('\n');
 		if (n > 0) {
 			uri = new URI(t.substring(0, n));
-			reporter.trace("dropTarget cleaned up from %s to %s", t, uri);
+			logger.debug("dropTarget cleaned up from {} to {}", t, uri);
 		}
 
 		if ("search.maven.org".equals(uri.getHost()) && "/remotecontent".equals(uri.getPath())) {
@@ -732,9 +737,6 @@ public class MavenBndRepository extends BaseRepository
 		if (uri.getPath() != null && uri.getPath().endsWith(".pom"))
 			return addPom(uri);
 
-		if (doJpm(uri))
-			return true;
-
 		return false;
 	}
 
@@ -748,14 +750,14 @@ public class MavenBndRepository extends BaseRepository
 	private boolean addPom(URI uri) throws Exception {
 		try {
 			// http://search.maven.org/remotecontent?filepath=com/netflix/governator/governator-commons-cli/1.12.10/governator-commons-cli-1.12.10.pom
-			IPom pom = storage.getPom(uri.toURL().openStream());
+			IPom pom = storage.getPom(client.connect(uri.toURL()));
 			Archive binaryArchive = pom.binaryArchive();
 			index.add(binaryArchive);
 			return true;
 		} catch (FileNotFoundException e) {
 			return false;
 		} catch (Exception e) {
-			e.printStackTrace();
+			logger.debug("Failure to parse {}", uri, e);
 			return false;
 		}
 	}
@@ -793,35 +795,6 @@ public class MavenBndRepository extends BaseRepository
 		return map;
 	}
 
-	JpmRepo getLibrary() throws Exception {
-		if (libraryx == null) {
-			HttpClient client = registry.getPlugin(HttpClient.class);
-			libraryx = JSONRPCProxy.createRPC(JpmRepo.class, client,
-					new URI("http://repo.jpm4j.org/" + JSONRPCProxy.JSONRPC_2_0 + "jpm"));
-		}
-		return libraryx;
-	}
-
-	boolean doJpm(URI uri) {
-		Matcher m = JPM_REVISION_URL_PATTERN_P.matcher(uri.toString());
-		if (m.matches())
-			try {
-				byte[] sha1 = Hex.toByteArray(m.group("sha"));
-				aQute.service.library.Library.Revision revision = getLibrary().getRevision(sha1);
-				if (revision != null) {
-					Archive a = Program.valueOf(revision.groupId, revision.artifactId)
-							.version(revision.version)
-							.archive("jar", revision.classifier);
-					index.add(a);
-					return true;
-				}
-
-			} catch (Exception e) {
-				reporter.exception(e, "Cannot drop JPM %s", uri);
-			}
-		return false;
-	}
-
 	@Override
 	public void toPom(OutputStream out, PomOptions options) throws Exception {
 		init();
@@ -830,7 +803,7 @@ public class MavenBndRepository extends BaseRepository
 				.parent(Revision.valueOf(options.parent))
 				.dependencyManagement(options.dependencyManagement)
 				.out(out);
-		
+
 	}
 
 	@Override
@@ -839,4 +812,20 @@ public class MavenBndRepository extends BaseRepository
 		return index.findProviders(requirements);
 	}
 
+	@Override
+	public void begin(Project project) {
+		releasePlugin = new ReleasePluginImpl(this, project);
+	}
+
+	@Override
+	public void end(Project p) {
+		System.out.println("Project ending is " + p);
+		try {
+			releasePlugin.end(p, storage);
+		} catch (Exception e) {
+			e.printStackTrace();
+			p.error("Could not end the release", e);
+		}
+	}
+
 }
