@@ -854,14 +854,14 @@ olsr_wallclock_string(void)
 {
   static char buf[sizeof("00:00:00.000000")];
   struct timeval now;
-  int sec, usec;
+  unsigned long sec, usec;
 
   gettimeofday(&now, NULL);
 
-  sec = (int)now.tv_sec + olsr_get_timezone();
-  usec = (int)now.tv_usec;
+  sec = (unsigned long) (now.tv_sec + olsr_get_timezone());
+  usec = (unsigned long) (now.tv_usec % 1000000);
 
-  snprintf(buf, sizeof(buf), "%02d:%02d:%02d.%06d", (sec % 86400) / 3600, (sec % 3600) / 60, sec % 60, usec);
+  snprintf(buf, sizeof(buf), "%02lu:%02lu:%02lu.%06lu", (sec % 86400) / 3600, (sec % 3600) / 60, sec % 60, usec);
 
   return buf;
 }
@@ -877,7 +877,7 @@ olsr_wallclock_string(void)
 const char *
 olsr_clock_string(uint32_t clk)
 {
-  static char buf[sizeof("00:00:00.000")];
+  static char buf[sizeof("00:00:00.000  ")];
 
   /* On most systems a clocktick is a 10ms quantity. */
   unsigned int msec = clk % 1000;
